/*
 * Licensed to Crate under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.  Crate licenses this file
 * to you under the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.  See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * However, if you have executed another commercial license agreement
 * with Crate these terms will supersede the license and you may use the
 * software solely pursuant to the terms of the relevant commercial
 * agreement.
 */

package io.crate.expression.operator;

import io.crate.data.Input;
import io.crate.expression.symbol.Function;
import io.crate.expression.symbol.Literal;
import io.crate.expression.symbol.Symbol;
import io.crate.metadata.Scalar;
import io.crate.metadata.TransactionContext;
import io.crate.types.BooleanType;
import io.crate.types.DataTypes;

public abstract class Operator<I> extends Scalar<Boolean, I> {

    public static final BooleanType RETURN_TYPE = DataTypes.BOOLEAN;
    public static final String PREFIX = "op_";

    @Override
    public Symbol normalizeSymbol(Function function, TransactionContext txnCtx) {
        // all operators evaluates to NULL if one argument is NULL
        // let's handle this here to prevent unnecessary collect operations
        for (Symbol symbol : function.arguments()) {
            if (symbol instanceof Input && ((Input<?>) symbol).value() == null) {
                return Literal.of(RETURN_TYPE, null);
            }
        }
        return super.normalizeSymbol(function, txnCtx);
    }
}
